from typing import Any, Optional

from django.contrib.messages.storage.base import BaseStorage
from django.http.request import HttpRequest

def default_storage(request: HttpRequest) -> BaseStorage: ...



import sys
from types import TracebackType
from typing import IO, Any, List, NamedTuple, Optional, Text, Tuple, Type, Union, overload
from typing_extensions import Literal

class Error(Exception): ...

class _aifc_params(NamedTuple):
    nchannels: int
    sampwidth: int
    framerate: int
    nframes: int
    comptype: bytes
    compname: bytes

_File = Union[Text, IO[bytes]]
_Marker = Tuple[int, int, bytes]

class Aifc_read:
    def __init__(self, f: _File) -> None: ...
    if sys.version_info >= (3, 4):
        def __enter__(self) -> Aifc_read: ...
        def __exit__(
            self, exc_type: Optional[Type[BaseException]], exc_val: Optional[BaseException], exc_tb: Optional[TracebackType]
        ) -> None: ...
    def initfp(self, file: IO[bytes]) -> None: ...
    def getfp(self) -> IO[bytes]: ...
    def rewind(self) -> None: ...
    def close(self) -> None: ...
    def tell(self) -> int: ...
    def getnchannels(self) -> int: ...
    def getnframes(self) -> int: ...
    def getsampwidth(self) -> int: ...
    def getframerate(self) -> int: ...
    def getcomptype(self) -> bytes: ...
    def getcompname(self) -> bytes: ...
    def getparams(self) -> _aifc_params: ...
    def getmarkers(self) -> Optional[List[_Marker]]: ...
    def getmark(self, id: int) -> _Marker: ...
    def setpos(self, pos: int) -> None: ...
    def readframes(self, nframes: int) -> bytes: ...

class Aifc_write:
    def __init__(self, f: _File) -> None: ...
    def __del__(self) -> None: ...
    if sys.version_info >= (3, 4):
        def __enter__(self) -> Aifc_write: ...
        def __exit__(
            self, exc_type: Optional[Type[BaseException]], exc_val: Optional[BaseException], exc_tb: Optional[TracebackType]
        ) -> None: ...
    def initfp(self, file: IO[bytes]) -> None: ...
    def aiff(self) -> None: ...
    def aifc(self) -> None: ...
    def setnchannels(self, nchannels: int) -> None: ...
    def getnchannels(self) -> int: ...
    def setsampwidth(self, sampwidth: int) -> None: ...
    def getsampwidth(self) -> int: ...
    def setframerate(self, framerate: int) -> None: ...
    def getframerate(self) -> int: ...
    def setnframes(self, nframes: int) -> None: ...
    def getnframes(self) -> int: ...
    def setcomptype(self, comptype: bytes, compname: bytes) -> None: ...
    def getcomptype(self) -> bytes: ...
    def getcompname(self) -> bytes: ...
    def setparams(self, params: Tuple[int, int, int, int, bytes, bytes]) -> None: ...
    def getparams(self) -> _aifc_params: ...
    def setmark(self, id: int, pos: int, name: bytes) -> None: ...
    def getmark(self, id: int) -> _Marker: ...
    def getmarkers(self) -> Optional[List[_Marker]]: ...
    def tell(self) -> int: ...
    def writeframesraw(self, data: Any) -> None: ...  # Actual type for data is Buffer Protocol
    def writeframes(self, data: Any) -> None: ...
    def close(self) -> None: ...

@overload
def open(f: _File, mode: Literal["r", "rb"]) -> Aifc_read: ...
@overload
def open(f: _File, mode: Literal["w", "wb"]) -> Aifc_write: ...
@overload
def open(f: _File, mode: Optional[str] = ...) -> Any: ...

if sys.version_info < (3, 9):
    @overload
    def openfp(f: _File, mode: Literal["r", "rb"]) -> Aifc_read: ...
    @overload
    def openfp(f: _File, mode: Literal["w", "wb"]) -> Aifc_write: ...
    @overload
    def openfp(f: _File, mode: Optional[str] = ...) -> Any: ...



import getpass as getpass  # noqa: F401
from typing import Any

from django.core.management.base import BaseCommand

class NotRunningInTTYException(Exception): ...

class Command(BaseCommand):
    stdin: Any



import email.message
from email.policy import Policy
from typing import Optional, Tuple, Union

_ParamsType = Union[str, None, Tuple[str, Optional[str], str]]

class MIMEBase(email.message.Message):
    def __init__(self, _maintype: str, _subtype: str, *, policy: Optional[Policy] = ..., **_params: _ParamsType) -> None: ...



from typing import Any, Callable, List, Mapping, Optional, Tuple

def get_platform() -> str: ...
def convert_path(pathname: str) -> str: ...
def change_root(new_root: str, pathname: str) -> str: ...
def check_environ() -> None: ...
def subst_vars(s: str, local_vars: Mapping[str, str]) -> None: ...
def split_quoted(s: str) -> List[str]: ...
def execute(
    func: Callable[..., None], args: Tuple[Any, ...], msg: Optional[str] = ..., verbose: bool = ..., dry_run: bool = ...
) -> None: ...
def strtobool(val: str) -> bool: ...
def byte_compile(
    py_files: List[str],
    optimize: int = ...,
    force: bool = ...,
    prefix: Optional[str] = ...,
    base_dir: Optional[str] = ...,
    verbose: bool = ...,
    dry_run: bool = ...,
    direct: Optional[bool] = ...,
) -> None: ...
def rfc822_escape(header: str) -> str: ...



import tkinter
from typing import Any, List, Optional, Tuple, TypeVar, Union, overload
from typing_extensions import Literal, TypedDict

NORMAL: Literal["normal"]
ROMAN: Literal["roman"]
BOLD: Literal["bold"]
ITALIC: Literal["italic"]

def nametofont(name: str) -> Font: ...

# See 'FONT DESCRIPTIONS' in font man page. This uses str because Literal
# inside Tuple doesn't work.
_FontDescription = Union[str, Font, Tuple[str, int], Tuple[str, int, str], Tuple[str, int, tkinter._TkinterSequence[str]]]

class _FontDict(TypedDict):
    family: str
    size: int
    weight: Literal["normal", "bold"]
    slant: Literal["roman", "italic"]
    underline: Literal[0, 1]
    overstrike: Literal[0, 1]

class _MetricsDict(TypedDict):
    ascent: int
    descent: int
    linespace: int
    fixed: Literal[0, 1]

class Font:
    name: str
    delete_font: bool
    def __init__(
        self,
        # In tkinter, 'root' refers to tkinter.Tk by convention, but the code
        # actually works with any tkinter widget so we use tkinter.Misc.
        root: Optional[tkinter.Misc] = ...,
        font: Optional[_FontDescription] = ...,
        name: Optional[str] = ...,
        exists: bool = ...,
        *,
        family: str = ...,
        size: int = ...,
        weight: Literal["normal", "bold"] = ...,
        slant: Literal["roman", "italic"] = ...,
        underline: bool = ...,
        overstrike: bool = ...,
    ) -> None: ...
    def __getitem__(self, key: str) -> Any: ...
    def __setitem__(self, key: str, value: Any) -> None: ...
    @overload
    def cget(self, option: Literal["family"]) -> str: ...
    @overload
    def cget(self, option: Literal["size"]) -> int: ...
    @overload
    def cget(self, option: Literal["weight"]) -> Literal["normal", "bold"]: ...
    @overload
    def cget(self, option: Literal["slant"]) -> Literal["roman", "italic"]: ...
    @overload
    def cget(self, option: Literal["underline", "overstrike"]) -> Literal[0, 1]: ...
    @overload
    def actual(self, option: Literal["family"], displayof: Optional[tkinter.Misc] = ...) -> str: ...
    @overload
    def actual(self, option: Literal["size"], displayof: Optional[tkinter.Misc] = ...) -> int: ...
    @overload
    def actual(self, option: Literal["weight"], displayof: Optional[tkinter.Misc] = ...) -> Literal["normal", "bold"]: ...
    @overload
    def actual(self, option: Literal["slant"], displayof: Optional[tkinter.Misc] = ...) -> Literal["roman", "italic"]: ...
    @overload
    def actual(self, option: Literal["underline", "overstrike"], displayof: Optional[tkinter.Misc] = ...) -> Literal[0, 1]: ...
    @overload
    def actual(self, option: None, displayof: Optional[tkinter.Misc] = ...) -> _FontDict: ...
    @overload
    def actual(self, *, displayof: Optional[tkinter.Misc] = ...) -> _FontDict: ...
    def config(
        self,
        *,
        family: str = ...,
        size: int = ...,
        weight: Literal["normal", "bold"] = ...,
        slant: Literal["roman", "italic"] = ...,
        underline: bool = ...,
        overstrike: bool = ...,
    ) -> Optional[_FontDict]: ...
    configure = config
    def copy(self) -> Font: ...
    @overload
    def metrics(self, __option: Literal["ascent", "descent", "linespace"], *, displayof: Optional[tkinter.Misc] = ...) -> int: ...
    @overload
    def metrics(self, __option: Literal["fixed"], *, displayof: Optional[tkinter.Misc] = ...) -> Literal[0, 1]: ...
    @overload
    def metrics(self, *, displayof: Optional[tkinter.Misc] = ...) -> _MetricsDict: ...
    def measure(self, text: str, displayof: Optional[tkinter.Misc] = ...) -> int: ...

def families(root: Optional[tkinter.Misc] = ..., displayof: Optional[tkinter.Misc] = ...) -> Tuple[str, ...]: ...
def names(root: Optional[tkinter.Misc] = ...) -> Tuple[str, ...]: ...



from typing import Any

from tornado import httputil
from tornado.tcpserver import TCPServer
from tornado.util import Configurable

class HTTPServer(TCPServer, Configurable, httputil.HTTPServerConnectionDelegate):
    def __init__(self, *args, **kwargs) -> None: ...
    request_callback: Any
    no_keep_alive: Any
    xheaders: Any
    protocol: Any
    conn_params: Any
    def initialize(
        self,
        request_callback,
        no_keep_alive=...,
        io_loop=...,
        xheaders=...,
        ssl_options=...,
        protocol=...,
        decompress_request=...,
        chunk_size=...,
        max_header_size=...,
        idle_connection_timeout=...,
        body_timeout=...,
        max_body_size=...,
        max_buffer_size=...,
    ): ...
    @classmethod
    def configurable_base(cls): ...
    @classmethod
    def configurable_default(cls): ...
    def close_all_connections(self): ...
    def handle_stream(self, stream, address): ...
    def start_request(self, server_conn, request_conn): ...
    def on_close(self, server_conn): ...

class _HTTPRequestContext:
    address: Any
    protocol: Any
    address_family: Any
    remote_ip: Any
    def __init__(self, stream, address, protocol) -> None: ...

class _ServerRequestAdapter(httputil.HTTPMessageDelegate):
    server: Any
    connection: Any
    request: Any
    delegate: Any
    def __init__(self, server, server_conn, request_conn) -> None: ...
    def headers_received(self, start_line, headers): ...
    def data_received(self, chunk): ...
    def finish(self): ...
    def on_connection_close(self): ...

HTTPRequest: Any



from typing import Any, Callable, Dict, List, Optional, Sequence, Set, Tuple, Type, TypeVar, Union, Collection, ClassVar

from django.core.checks.messages import CheckMessage
from django.core.exceptions import ValidationError
from django.db.models.manager import BaseManager
from django.db.models.options import Options

_Self = TypeVar("_Self", bound="Model")

class ModelStateFieldsCacheDescriptor: ...

class ModelState:
    db: Optional[str] = ...
    adding: bool = ...
    fields_cache: ModelStateFieldsCacheDescriptor = ...

class ModelBase(type): ...

class Model(metaclass=ModelBase):
    class DoesNotExist(Exception): ...
    class MultipleObjectsReturned(Exception): ...
    class Meta: ...
    _meta: Options[Any]
    _default_manager: BaseManager[Model]
    objects: ClassVar[BaseManager[Any]]
    pk: Any = ...
    _state: ModelState
    def __init__(self: _Self, *args, **kwargs) -> None: ...
    @classmethod
    def add_to_class(cls, name: str, value: Any): ...
    @classmethod
    def from_db(cls, db: Optional[str], field_names: Collection[str], values: Collection[Any]) -> _Self: ...
    def delete(self, using: Any = ..., keep_parents: bool = ...) -> Tuple[int, Dict[str, int]]: ...
    def full_clean(self, exclude: Optional[Collection[str]] = ..., validate_unique: bool = ...) -> None: ...
    def clean(self) -> None: ...
    def clean_fields(self, exclude: Optional[Collection[str]] = ...) -> None: ...
    def validate_unique(self, exclude: Optional[Collection[str]] = ...) -> None: ...
    def unique_error_message(
        self, model_class: Type[_Self], unique_check: Collection[Union[Callable, str]]
    ) -> ValidationError: ...
    def save(
        self,
        force_insert: bool = ...,
        force_update: bool = ...,
        using: Optional[str] = ...,
        update_fields: Optional[Union[Sequence[str], str]] = ...,
    ) -> None: ...
    def save_base(
        self,
        raw: bool = ...,
        force_insert: bool = ...,
        force_update: bool = ...,
        using: Optional[str] = ...,
        update_fields: Optional[Union[Sequence[str], str]] = ...,
    ): ...
    def refresh_from_db(self: _Self, using: Optional[str] = ..., fields: Optional[List[str]] = ...) -> None: ...
    def get_deferred_fields(self) -> Set[str]: ...
    @classmethod
    def check(cls, **kwargs: Any) -> List[CheckMessage]: ...
    def __getstate__(self) -> dict: ...



from typing import Any

from tornado.util import Configurable

class HTTPClient:
    def __init__(self, async_client_class=..., **kwargs) -> None: ...
    def __del__(self): ...
    def close(self): ...
    def fetch(self, request, **kwargs): ...

class AsyncHTTPClient(Configurable):
    @classmethod
    def configurable_base(cls): ...
    @classmethod
    def configurable_default(cls): ...
    def __new__(cls, io_loop=..., force_instance=..., **kwargs): ...
    io_loop: Any
    defaults: Any
    def initialize(self, io_loop, defaults=...): ...
    def close(self): ...
    def fetch(self, request, callback=..., raise_error=..., **kwargs): ...
    def fetch_impl(self, request, callback): ...
    @classmethod
    def configure(cls, impl, **kwargs): ...

class HTTPRequest:
    proxy_host: Any
    proxy_port: Any
    proxy_username: Any
    proxy_password: Any
    url: Any
    method: Any
    body_producer: Any
    auth_username: Any
    auth_password: Any
    auth_mode: Any
    connect_timeout: Any
    request_timeout: Any
    follow_redirects: Any
    max_redirects: Any
    user_agent: Any
    decompress_response: Any
    network_interface: Any
    streaming_callback: Any
    header_callback: Any
    prepare_curl_callback: Any
    allow_nonstandard_methods: Any
    validate_cert: Any
    ca_certs: Any
    allow_ipv6: Any
    client_key: Any
    client_cert: Any
    ssl_options: Any
    expect_100_continue: Any
    start_time: Any
    def __init__(
        self,
        url,
        method=...,
        headers=...,
        body=...,
        auth_username=...,
        auth_password=...,
        auth_mode=...,
        connect_timeout=...,
        request_timeout=...,
        if_modified_since=...,
        follow_redirects=...,
        max_redirects=...,
        user_agent=...,
        use_gzip=...,
        network_interface=...,
        streaming_callback=...,
        header_callback=...,
        prepare_curl_callback=...,
        proxy_host=...,
        proxy_port=...,
        proxy_username=...,
        proxy_password=...,
        allow_nonstandard_methods=...,
        validate_cert=...,
        ca_certs=...,
        allow_ipv6=...,
        client_key=...,
        client_cert=...,
        body_producer=...,
        expect_100_continue=...,
        decompress_response=...,
        ssl_options=...,
    ) -> None: ...
    @property
    def headers(self): ...
    @headers.setter
    def headers(self, value): ...
    @property
    def body(self): ...
    @body.setter
    def body(self, value): ...

class HTTPResponse:
    request: Any
    code: Any
    reason: Any
    headers: Any
    buffer: Any
    effective_url: Any
    error: Any
    request_time: Any
    time_info: Any
    def __init__(
        self, request, code, headers=..., buffer=..., effective_url=..., error=..., request_time=..., time_info=..., reason=...
    ) -> None: ...
    body: bytes
    def rethrow(self): ...

class HTTPError(Exception):
    code: Any
    response: Any
    def __init__(self, code, message=..., response=...) -> None: ...

class _RequestProxy:
    request: Any
    defaults: Any
    def __init__(self, request, defaults) -> None: ...
    def __getattr__(self, name): ...

def main(): ...



import sys
from typing import Any, Optional, Text

if sys.version_info >= (3,):
    from io import BytesIO as BytesIO, StringIO as StringIO

    NativeStringIO = StringIO
else:
    import cStringIO
    from StringIO import StringIO as StringIO

    BytesIO = cStringIO.StringIO
    NativeStringIO = BytesIO

PY2: Any
WIN: Any
unichr: Any
text_type: Any
string_types: Any
integer_types: Any
iterkeys: Any
itervalues: Any
iteritems: Any
iterlists: Any
iterlistvalues: Any
int_to_byte: Any
iter_bytes: Any

def fix_tuple_repr(obj): ...
def implements_iterator(cls): ...
def implements_to_string(cls): ...
def native_string_result(func): ...
def implements_bool(cls): ...

range_type: Any

def make_literal_wrapper(reference): ...
def normalize_string_tuple(tup): ...
def try_coerce_native(s): ...

wsgi_get_bytes: Any

def wsgi_decoding_dance(s, charset: Text = ..., errors: Text = ...): ...
def wsgi_encoding_dance(s, charset: Text = ..., errors: Text = ...): ...
def to_bytes(x, charset: Text = ..., errors: Text = ...): ...
def to_native(x, charset: Text = ..., errors: Text = ...): ...
def reraise(tp, value, tb: Optional[Any] = ...): ...

imap: Any
izip: Any
ifilter: Any

def to_unicode(x, charset: Text = ..., errors: Text = ..., allow_none_charset: bool = ...): ...



from typing import Any, Optional, Tuple

__copyright__: Any
DEV_NULL: Any

def libc_ver(executable=..., lib=..., version=..., chunksize: int = ...): ...
def linux_distribution(distname=..., version=..., id=..., supported_dists=..., full_distribution_name: int = ...): ...
def dist(distname=..., version=..., id=..., supported_dists=...): ...

class _popen:
    tmpfile: Any
    pipe: Any
    bufsize: Any
    mode: Any
    def __init__(self, cmd, mode=..., bufsize: Optional[Any] = ...): ...
    def read(self): ...
    def readlines(self): ...
    def close(self, remove=..., error=...): ...
    __del__: Any

def popen(cmd, mode=..., bufsize: Optional[Any] = ...): ...
def win32_ver(release=..., version=..., csd=..., ptype=...): ...
def mac_ver(release=..., versioninfo=..., machine=...): ...
def java_ver(release=..., vendor=..., vminfo=..., osinfo=...): ...
def system_alias(system, release, version): ...
def architecture(executable=..., bits=..., linkage=...) -> Tuple[str, str]: ...
def uname() -> Tuple[str, str, str, str, str, str]: ...
def system() -> str: ...
def node() -> str: ...
def release() -> str: ...
def version() -> str: ...
def machine() -> str: ...
def processor() -> str: ...
def python_implementation() -> str: ...
def python_version() -> str: ...
def python_version_tuple() -> Tuple[str, str, str]: ...
def python_branch() -> str: ...
def python_revision() -> str: ...
def python_build() -> Tuple[str, str]: ...
def python_compiler() -> str: ...
def platform(aliased: int = ..., terse: int = ...) -> str: ...



import sys
from typing import Any, Callable, Dict, List, Optional

from tornado import httputil

MIN_SUPPORTED_SIGNED_VALUE_VERSION: Any
MAX_SUPPORTED_SIGNED_VALUE_VERSION: Any
DEFAULT_SIGNED_VALUE_VERSION: Any
DEFAULT_SIGNED_VALUE_MIN_VERSION: Any

if sys.version_info >= (3, 5):
    from typing import Awaitable

    _MethodType = Callable[..., Optional[Awaitable[None]]]
else:
    _MethodType = Callable[..., Any]

class RequestHandler:
    SUPPORTED_METHODS: Any
    application: Application
    request: httputil.HTTPServerRequest
    path_args: List[str]
    path_kwargs: Dict[str, str]
    ui: Any
    def __init__(self, application, request, **kwargs) -> None: ...
    initialize: Callable[..., None] = ...
    @property
    def settings(self): ...
    head: _MethodType
    get: _MethodType
    post: _MethodType
    delete: _MethodType
    patch: _MethodType
    put: _MethodType
    options: _MethodType
    def prepare(self): ...
    def on_finish(self): ...
    def on_connection_close(self): ...
    def clear(self): ...
    def set_default_headers(self): ...
    def set_status(self, status_code, reason=...): ...
    def get_status(self): ...
    def set_header(self, name, value): ...
    def add_header(self, name, value): ...
    def clear_header(self, name): ...
    def get_argument(self, name, default=..., strip=...): ...
    def get_arguments(self, name, strip=...): ...
    def get_body_argument(self, name, default=..., strip=...): ...
    def get_body_arguments(self, name, strip=...): ...
    def get_query_argument(self, name, default=..., strip=...): ...
    def get_query_arguments(self, name, strip=...): ...
    def decode_argument(self, value, name=...): ...
    @property
    def cookies(self): ...
    def get_cookie(self, name, default=...): ...
    def set_cookie(self, name, value, domain=..., expires=..., path=..., expires_days=..., **kwargs): ...
    def clear_cookie(self, name, path=..., domain=...): ...
    def clear_all_cookies(self, path=..., domain=...): ...
    def set_secure_cookie(self, name, value, expires_days=..., version=..., **kwargs): ...
    def create_signed_value(self, name, value, version=...): ...
    def get_secure_cookie(self, name, value=..., max_age_days=..., min_version=...): ...
    def get_secure_cookie_key_version(self, name, value=...): ...
    def redirect(self, url, permanent=..., status=...): ...
    def write(self, chunk): ...
    def render(self, template_name, **kwargs): ...
    def render_string(self, template_name, **kwargs): ...
    def get_template_namespace(self): ...
    def create_template_loader(self, template_path): ...
    def flush(self, include_footers=..., callback=...): ...
    def finish(self, chunk=...): ...
    def send_error(self, status_code=..., **kwargs): ...
    def write_error(self, status_code, **kwargs): ...
    @property
    def locale(self): ...
    @locale.setter
    def locale(self, value): ...
    def get_user_locale(self): ...
    def get_browser_locale(self, default=...): ...
    @property
    def current_user(self): ...
    @current_user.setter
    def current_user(self, value): ...
    def get_current_user(self): ...
    def get_login_url(self): ...
    def get_template_path(self): ...
    @property
    def xsrf_token(self): ...
    def check_xsrf_cookie(self): ...
    def xsrf_form_html(self): ...
    def static_url(self, path, include_host=..., **kwargs): ...
    def require_setting(self, name, feature=...): ...
    def reverse_url(self, name, *args): ...
    def compute_etag(self): ...
    def set_etag_header(self): ...
    def check_etag_header(self): ...
    def data_received(self, chunk): ...
    def log_exception(self, typ, value, tb): ...

def asynchronous(method): ...
def stream_request_body(cls): ...
def removeslash(method): ...
def addslash(method): ...

class Application(httputil.HTTPServerConnectionDelegate):
    transforms: Any
    handlers: Any
    named_handlers: Any
    default_host: Any
    settings: Any
    ui_modules: Any
    ui_methods: Any
    def __init__(self, handlers=..., default_host=..., transforms=..., **settings) -> None: ...
    def listen(self, port, address=..., **kwargs): ...
    def add_handlers(self, host_pattern, host_handlers): ...
    def add_transform(self, transform_class): ...
    def start_request(self, server_conn, request_conn): ...
    def __call__(self, request): ...
    def reverse_url(self, name, *args): ...
    def log_request(self, handler): ...

class _RequestDispatcher(httputil.HTTPMessageDelegate):
    application: Any
    connection: Any
    request: Any
    chunks: Any
    handler_class: Any
    handler_kwargs: Any
    path_args: Any
    path_kwargs: Any
    def __init__(self, application, connection) -> None: ...
    def headers_received(self, start_line, headers): ...
    stream_request_body: Any
    def set_request(self, request): ...
    def data_received(self, data): ...
    def finish(self): ...
    def on_connection_close(self): ...
    handler: Any
    def execute(self): ...

class HTTPError(Exception):
    status_code: Any
    log_message: Any
    args: Any
    reason: Any
    def __init__(self, status_code, log_message=..., *args, **kwargs) -> None: ...

class Finish(Exception): ...

class MissingArgumentError(HTTPError):
    arg_name: Any
    def __init__(self, arg_name) -> None: ...

class ErrorHandler(RequestHandler):
    def initialize(self, status_code): ...
    def prepare(self): ...
    def check_xsrf_cookie(self): ...

class RedirectHandler(RequestHandler):
    def initialize(self, url, permanent=...): ...
    def get(self): ...

class StaticFileHandler(RequestHandler):
    CACHE_MAX_AGE: Any
    root: Any
    default_filename: Any
    def initialize(self, path, default_filename=...): ...
    @classmethod
    def reset(cls): ...
    def head(self, path): ...
    path: Any
    absolute_path: Any
    modified: Any
    def get(self, path, include_body=...): ...
    def compute_etag(self): ...
    def set_headers(self): ...
    def should_return_304(self): ...
    @classmethod
    def get_absolute_path(cls, root, path): ...
    def validate_absolute_path(self, root, absolute_path): ...
    @classmethod
    def get_content(cls, abspath, start=..., end=...): ...
    @classmethod
    def get_content_version(cls, abspath): ...
    def get_content_size(self): ...
    def get_modified_time(self): ...
    def get_content_type(self): ...
    def set_extra_headers(self, path): ...
    def get_cache_time(self, path, modified, mime_type): ...
    @classmethod
    def make_static_url(cls, settings, path, include_version=...): ...
    def parse_url_path(self, url_path): ...
    @classmethod
    def get_version(cls, settings, path): ...

class FallbackHandler(RequestHandler):
    fallback: Any
    def initialize(self, fallback): ...
    def prepare(self): ...

class OutputTransform:
    def __init__(self, request) -> None: ...
    def transform_first_chunk(self, status_code, headers, chunk, finishing): ...
    def transform_chunk(self, chunk, finishing): ...

class GZipContentEncoding(OutputTransform):
    CONTENT_TYPES: Any
    MIN_LENGTH: Any
    def __init__(self, request) -> None: ...
    def transform_first_chunk(self, status_code, headers, chunk, finishing): ...
    def transform_chunk(self, chunk, finishing): ...

def authenticated(method): ...

class UIModule:
    handler: Any
    request: Any
    ui: Any
    locale: Any
    def __init__(self, handler) -> None: ...
    @property
    def current_user(self): ...
    def render(self, *args, **kwargs): ...
    def embedded_javascript(self): ...
    def javascript_files(self): ...
    def embedded_css(self): ...
    def css_files(self): ...
    def html_head(self): ...
    def html_body(self): ...
    def render_string(self, path, **kwargs): ...

class _linkify(UIModule):
    def render(self, text, **kwargs): ...

class _xsrf_form_html(UIModule):
    def render(self): ...

class TemplateModule(UIModule):
    def __init__(self, handler) -> None: ...
    def render(self, path, **kwargs): ...
    def embedded_javascript(self): ...
    def javascript_files(self): ...
    def embedded_css(self): ...
    def css_files(self): ...
    def html_head(self): ...
    def html_body(self): ...

class _UIModuleNamespace:
    handler: Any
    ui_modules: Any
    def __init__(self, handler, ui_modules) -> None: ...
    def __getitem__(self, key): ...
    def __getattr__(self, key): ...

class URLSpec:
    regex: Any
    handler_class: Any
    kwargs: Any
    name: Any
    def __init__(self, pattern, handler, kwargs=..., name=...) -> None: ...
    def reverse(self, *args): ...

url: Any

def create_signed_value(secret, name, value, version=..., clock=..., key_version=...): ...
def decode_signed_value(secret, name, value, max_age_days=..., clock=..., min_version=...): ...
def get_signature_key_version(value): ...



from typing import Dict, Iterator, List, Set

from django.db.migrations.operations.base import Operation

def topological_sort_as_sets(dependency_graph: Dict[Operation, Set[Operation]]) -> Iterator[Set[Operation]]: ...
def stable_topological_sort(
    l: List[Operation], dependency_graph: Dict[Operation, Set[Operation]]
) -> List[Operation]: ...



from types import ClassType, FrameType, ModuleType, TracebackType
from typing import IO, Any, BinaryIO, Callable, Dict, List, NoReturn, Optional, Sequence, Text, Tuple, Type, Union, overload

# The following type alias are stub-only and do not exist during runtime
_ExcInfo = Tuple[Type[BaseException], BaseException, TracebackType]
_OptExcInfo = Union[_ExcInfo, Tuple[None, None, None]]

class _flags:
    bytes_warning: int
    debug: int
    division_new: int
    division_warning: int
    dont_write_bytecode: int
    hash_randomization: int
    ignore_environment: int
    inspect: int
    interactive: int
    no_site: int
    no_user_site: int
    optimize: int
    py3k_warning: int
    tabcheck: int
    unicode: int
    verbose: int

class _float_info:
    max: float
    max_exp: int
    max_10_exp: int
    min: float
    min_exp: int
    min_10_exp: int
    dig: int
    mant_dig: int
    epsilon: float
    radix: int
    rounds: int

class _version_info(Tuple[int, int, int, str, int]):
    major: int
    minor: int
    micro: int
    releaselevel: str
    serial: int

_mercurial: Tuple[str, str, str]
api_version: int
argv: List[str]
builtin_module_names: Tuple[str, ...]
byteorder: str
copyright: str
dont_write_bytecode: bool
exec_prefix: str
executable: str
flags: _flags
float_repr_style: str
hexversion: int
long_info: object
maxint: int
maxsize: int
maxunicode: int
modules: Dict[str, Any]
path: List[str]
platform: str
prefix: str
py3kwarning: bool
__stderr__: IO[str]
__stdin__: IO[str]
__stdout__: IO[str]
stderr: IO[str]
stdin: IO[str]
stdout: IO[str]
subversion: Tuple[str, str, str]
version: str
warnoptions: object
float_info: _float_info
version_info: _version_info
ps1: str
ps2: str
last_type: type
last_value: BaseException
last_traceback: TracebackType
# TODO precise types
meta_path: List[Any]
path_hooks: List[Any]
path_importer_cache: Dict[str, Any]
displayhook: Callable[[object], Any]
excepthook: Callable[[Type[BaseException], BaseException, TracebackType], Any]
exc_type: Optional[type]
exc_value: Union[BaseException, ClassType]
exc_traceback: TracebackType

class _WindowsVersionType:
    major: Any
    minor: Any
    build: Any
    platform: Any
    service_pack: Any
    service_pack_major: Any
    service_pack_minor: Any
    suite_mask: Any
    product_type: Any

def getwindowsversion() -> _WindowsVersionType: ...
def _clear_type_cache() -> None: ...
def _current_frames() -> Dict[int, FrameType]: ...
def _getframe(depth: int = ...) -> FrameType: ...
def call_tracing(fn: Any, args: Any) -> Any: ...
def __displayhook__(value: object) -> None: ...
def __excepthook__(type_: type, value: BaseException, traceback: TracebackType) -> None: ...
def exc_clear() -> None: ...
def exc_info() -> _OptExcInfo: ...

# sys.exit() accepts an optional argument of anything printable
def exit(arg: Any = ...) -> NoReturn: ...
def getcheckinterval() -> int: ...  # deprecated
def getdefaultencoding() -> str: ...
def getdlopenflags() -> int: ...
def getfilesystemencoding() -> str: ...  # In practice, never returns None
def getrefcount(arg: Any) -> int: ...
def getrecursionlimit() -> int: ...
def getsizeof(obj: object, default: int = ...) -> int: ...
def getprofile() -> Optional[Any]: ...
def gettrace() -> Optional[Any]: ...
def setcheckinterval(interval: int) -> None: ...  # deprecated
def setdlopenflags(n: int) -> None: ...
def setdefaultencoding(encoding: Text) -> None: ...  # only exists after reload(sys)
def setprofile(profilefunc: Any) -> None: ...  # TODO type
def setrecursionlimit(limit: int) -> None: ...
def settrace(tracefunc: Any) -> None: ...  # TODO type



from _typeshed import AnyPath
from typing import IO, Any, Callable, Iterable, Optional, Protocol, Text, TypeVar, Union
from typing_extensions import Literal

from markupsafe import Markup as Markup, escape as escape, soft_unicode as soft_unicode

missing: Any
internal_code: Any
concat: Any

_CallableT = TypeVar("_CallableT", bound=Callable[..., Any])

class _ContextFunction(Protocol[_CallableT]):
    contextfunction: Literal[True]
    __call__: _CallableT

class _EvalContextFunction(Protocol[_CallableT]):
    evalcontextfunction: Literal[True]
    __call__: _CallableT

class _EnvironmentFunction(Protocol[_CallableT]):
    environmentfunction: Literal[True]
    __call__: _CallableT

def contextfunction(f: _CallableT) -> _ContextFunction[_CallableT]: ...
def evalcontextfunction(f: _CallableT) -> _EvalContextFunction[_CallableT]: ...
def environmentfunction(f: _CallableT) -> _EnvironmentFunction[_CallableT]: ...
def internalcode(f: _CallableT) -> _CallableT: ...
def is_undefined(obj: object) -> bool: ...
def select_autoescape(
    enabled_extensions: Iterable[str] = ...,
    disabled_extensions: Iterable[str] = ...,
    default_for_string: bool = ...,
    default: bool = ...,
) -> Callable[[str], bool]: ...
def consume(iterable: Iterable[object]) -> None: ...
def clear_caches() -> None: ...
def import_string(import_name: str, silent: bool = ...) -> Any: ...
def open_if_exists(filename: AnyPath, mode: str = ...) -> Optional[IO[Any]]: ...
def object_type_repr(obj: object) -> str: ...
def pformat(obj: object, verbose: bool = ...) -> str: ...
def urlize(
    text: Union[Markup, Text],
    trim_url_limit: Optional[int] = ...,
    rel: Optional[Union[Markup, Text]] = ...,
    target: Optional[Union[Markup, Text]] = ...,
) -> str: ...
def generate_lorem_ipsum(n: int = ..., html: bool = ..., min: int = ..., max: int = ...) -> Union[Markup, str]: ...
def unicode_urlencode(obj: object, charset: str = ..., for_qs: bool = ...) -> str: ...

class LRUCache:
    capacity: Any
    def __init__(self, capacity) -> None: ...
    def __getnewargs__(self): ...
    def copy(self): ...
    def get(self, key, default: Optional[Any] = ...): ...
    def setdefault(self, key, default: Optional[Any] = ...): ...
    def clear(self): ...
    def __contains__(self, key): ...
    def __len__(self): ...
    def __getitem__(self, key): ...
    def __setitem__(self, key, value): ...
    def __delitem__(self, key): ...
    def items(self): ...
    def iteritems(self): ...
    def values(self): ...
    def itervalue(self): ...
    def keys(self): ...
    def iterkeys(self): ...
    __iter__: Any
    def __reversed__(self): ...
    __copy__: Any

class Cycler:
    items: Any
    def __init__(self, *items) -> None: ...
    pos: int
    def reset(self): ...
    @property
    def current(self): ...
    def __next__(self): ...

class Joiner:
    sep: Any
    used: bool
    def __init__(self, sep: str = ...) -> None: ...
    def __call__(self): ...



from typing import Any, List, Union, Iterable, Optional

from django.contrib.admin.options import BaseModelAdmin
from django.core.checks.messages import Error

from django.apps.config import AppConfig

_CheckError = Union[str, Error]

def check_admin_app(app_configs: Optional[Iterable[AppConfig]], **kwargs: Any) -> List[_CheckError]: ...
def check_dependencies(**kwargs: Any) -> List[_CheckError]: ...

class BaseModelAdminChecks:
    def check(self, admin_obj: BaseModelAdmin, **kwargs: Any) -> List[_CheckError]: ...

class ModelAdminChecks(BaseModelAdminChecks): ...
class InlineModelAdminChecks(BaseModelAdminChecks): ...

def must_be(type: Any, option: Any, obj: Any, id: Any): ...
def must_inherit_from(parent: Any, option: Any, obj: Any, id: Any): ...
def refer_to_missing_field(field: Any, option: Any, model: Any, obj: Any, id: Any): ...



def import_module(callback):
    """
    Handle "magic" Flask extension imports:
    ``flask.ext.foo`` is really ``flask_foo`` or ``flaskext.foo``.
    """
    def wrapper(inference_state, import_names, module_context, *args, **kwargs):
        if len(import_names) == 3 and import_names[:2] == ('flask', 'ext'):
            # New style.
            ipath = ('flask_' + import_names[2]),
            value_set = callback(inference_state, ipath, None, *args, **kwargs)
            if value_set:
                return value_set
            value_set = callback(inference_state, ('flaskext',), None, *args, **kwargs)
            return callback(
                inference_state,
                ('flaskext', import_names[2]),
                next(iter(value_set)),
                *args, **kwargs
            )
        return callback(inference_state, import_names, module_context, *args, **kwargs)
    return wrapper



from typing import IO, Any, AnyStr, Generic, Iterator, List, Optional, Text, TypeVar

_T = TypeVar("_T")

def _posixify(name: str) -> str: ...
def safecall(func: _T) -> _T: ...
def make_str(value: Any) -> str: ...
def make_default_short_help(help: str, max_length: int = ...): ...

class LazyFile(object):
    name: str
    mode: str
    encoding: Optional[str]
    errors: str
    atomic: bool
    def __init__(
        self, filename: str, mode: str = ..., encoding: Optional[str] = ..., errors: str = ..., atomic: bool = ...
    ) -> None: ...
    def open(self) -> IO[Any]: ...
    def close(self) -> None: ...
    def close_intelligently(self) -> None: ...
    def __enter__(self) -> LazyFile: ...
    def __exit__(self, exc_type, exc_value, tb): ...
    def __iter__(self) -> Iterator[Any]: ...

class KeepOpenFile(Generic[AnyStr]):
    _file: IO[AnyStr]
    def __init__(self, file: IO[AnyStr]) -> None: ...
    def __enter__(self) -> KeepOpenFile[AnyStr]: ...
    def __exit__(self, exc_type, exc_value, tb): ...
    def __iter__(self) -> Iterator[AnyStr]: ...

def echo(
    message: object = ..., file: Optional[IO[Text]] = ..., nl: bool = ..., err: bool = ..., color: Optional[bool] = ...
) -> None: ...
def get_binary_stream(name: str) -> IO[bytes]: ...
def get_text_stream(name: str, encoding: Optional[str] = ..., errors: str = ...) -> IO[str]: ...
def open_file(
    filename: str, mode: str = ..., encoding: Optional[str] = ..., errors: str = ..., lazy: bool = ..., atomic: bool = ...
) -> Any: ...  # really Union[IO, LazyFile, KeepOpenFile]
def get_os_args() -> List[str]: ...
def format_filename(filename: str, shorten: bool = ...) -> str: ...
def get_app_dir(app_name: str, roaming: bool = ..., force_posix: bool = ...) -> str: ...



from typing import Optional, Text

class GeoIP2Error(RuntimeError): ...
class AddressNotFoundError(GeoIP2Error): ...
class AuthenticationError(GeoIP2Error): ...

class HTTPError(GeoIP2Error):
    http_status: Optional[int]
    uri: Optional[Text]
    def __init__(self, message: Text, http_status: Optional[int] = ..., uri: Optional[Text] = ...) -> None: ...

class InvalidRequestError(GeoIP2Error): ...
class OutOfQueriesError(GeoIP2Error): ...
class PermissionRequiredError(GeoIP2Error): ...



import io
import sys
from _typeshed import AnyPath
from typing import IO, Any, Optional, TextIO, TypeVar, Union, overload
from typing_extensions import Literal

_PathOrFile = Union[AnyPath, IO[bytes]]
_T = TypeVar("_T")

def compress(data: bytes, compresslevel: int = ...) -> bytes: ...
def decompress(data: bytes) -> bytes: ...

if sys.version_info >= (3, 3):
    _OpenBinaryMode = Literal["r", "rb", "w", "wb", "x", "xb", "a", "ab"]
    _OpenTextMode = Literal["rt", "wt", "xt", "at"]
    @overload
    def open(
        filename: _PathOrFile,
        mode: _OpenBinaryMode = ...,
        compresslevel: int = ...,
        encoding: None = ...,
        errors: None = ...,
        newline: None = ...,
    ) -> BZ2File: ...
    @overload
    def open(
        filename: AnyPath,
        mode: _OpenTextMode,
        compresslevel: int = ...,
        encoding: Optional[str] = ...,
        errors: Optional[str] = ...,
        newline: Optional[str] = ...,
    ) -> TextIO: ...
    @overload
    def open(
        filename: _PathOrFile,
        mode: str,
        compresslevel: int = ...,
        encoding: Optional[str] = ...,
        errors: Optional[str] = ...,
        newline: Optional[str] = ...,
    ) -> Union[BZ2File, TextIO]: ...

class BZ2File(io.BufferedIOBase, IO[bytes]):
    def __enter__(self: _T) -> _T: ...
    if sys.version_info >= (3, 9):
        def __init__(self, filename: _PathOrFile, mode: str = ..., *, compresslevel: int = ...) -> None: ...
    else:
        def __init__(
            self, filename: _PathOrFile, mode: str = ..., buffering: Optional[Any] = ..., compresslevel: int = ...
        ) -> None: ...

class BZ2Compressor(object):
    def __init__(self, compresslevel: int = ...) -> None: ...
    def compress(self, data: bytes) -> bytes: ...
    def flush(self) -> bytes: ...

class BZ2Decompressor(object):
    if sys.version_info >= (3, 5):
        def decompress(self, data: bytes, max_length: int = ...) -> bytes: ...
    else:
        def decompress(self, data: bytes) -> bytes: ...
    if sys.version_info >= (3, 3):
        @property
        def eof(self) -> bool: ...
    if sys.version_info >= (3, 5):
        @property
        def needs_input(self) -> bool: ...
    @property
    def unused_data(self) -> bytes: ...



from typing import Any

from . import connection, exceptions, request, response
from .connection import BaseSSLError as BaseSSLError, ConnectionError as ConnectionError, HTTPException as HTTPException
from .packages import ssl_match_hostname
from .util import connection as _connection, retry, timeout, url

ClosedPoolError = exceptions.ClosedPoolError
ProtocolError = exceptions.ProtocolError
EmptyPoolError = exceptions.EmptyPoolError
HostChangedError = exceptions.HostChangedError
LocationValueError = exceptions.LocationValueError
MaxRetryError = exceptions.MaxRetryError
ProxyError = exceptions.ProxyError
ReadTimeoutError = exceptions.ReadTimeoutError
SSLError = exceptions.SSLError
TimeoutError = exceptions.TimeoutError
InsecureRequestWarning = exceptions.InsecureRequestWarning
CertificateError = ssl_match_hostname.CertificateError
port_by_scheme = connection.port_by_scheme
DummyConnection = connection.DummyConnection
HTTPConnection = connection.HTTPConnection
HTTPSConnection = connection.HTTPSConnection
VerifiedHTTPSConnection = connection.VerifiedHTTPSConnection
RequestMethods = request.RequestMethods
HTTPResponse = response.HTTPResponse
is_connection_dropped = _connection.is_connection_dropped
Retry = retry.Retry
Timeout = timeout.Timeout
get_host = url.get_host

xrange: Any
log: Any

class ConnectionPool:
    scheme: Any
    QueueCls: Any
    host: Any
    port: Any
    def __init__(self, host, port=...) -> None: ...
    def __enter__(self): ...
    def __exit__(self, exc_type, exc_val, exc_tb): ...
    def close(self): ...

class HTTPConnectionPool(ConnectionPool, RequestMethods):
    scheme: Any
    ConnectionCls: Any
    strict: Any
    timeout: Any
    retries: Any
    pool: Any
    block: Any
    proxy: Any
    proxy_headers: Any
    num_connections: Any
    num_requests: Any
    conn_kw: Any
    def __init__(
        self,
        host,
        port=...,
        strict=...,
        timeout=...,
        maxsize=...,
        block=...,
        headers=...,
        retries=...,
        _proxy=...,
        _proxy_headers=...,
        **conn_kw,
    ) -> None: ...
    def close(self): ...
    def is_same_host(self, url): ...
    def urlopen(
        self,
        method,
        url,
        body=...,
        headers=...,
        retries=...,
        redirect=...,
        assert_same_host=...,
        timeout=...,
        pool_timeout=...,
        release_conn=...,
        **response_kw,
    ): ...

class HTTPSConnectionPool(HTTPConnectionPool):
    scheme: Any
    ConnectionCls: Any
    key_file: Any
    cert_file: Any
    cert_reqs: Any
    ca_certs: Any
    ssl_version: Any
    assert_hostname: Any
    assert_fingerprint: Any
    def __init__(
        self,
        host,
        port=...,
        strict=...,
        timeout=...,
        maxsize=...,
        block=...,
        headers=...,
        retries=...,
        _proxy=...,
        _proxy_headers=...,
        key_file=...,
        cert_file=...,
        cert_reqs=...,
        ca_certs=...,
        ssl_version=...,
        assert_hostname=...,
        assert_fingerprint=...,
        **conn_kw,
    ) -> None: ...

def connection_from_url(url, **kw): ...



import os
from pathlib import Path
from typing import Optional

from jedi.inference.cache import inference_state_method_cache
from jedi.inference.names import AbstractNameDefinition, ModuleName
from jedi.inference.filters import GlobalNameFilter, ParserTreeFilter, DictFilter, MergedFilter
from jedi.inference import compiled
from jedi.inference.base_value import TreeValue
from jedi.inference.names import SubModuleName
from jedi.inference.helpers import values_from_qualified_names
from jedi.inference.compiled import create_simple_object
from jedi.inference.base_value import ValueSet
from jedi.inference.context import ModuleContext


class _ModuleAttributeName(AbstractNameDefinition):
    """
    For module attributes like __file__, __str__ and so on.
    """
    api_type = 'instance'

    def __init__(self, parent_module, string_name, string_value=None):
        self.parent_context = parent_module
        self.string_name = string_name
        self._string_value = string_value

    def infer(self):
        if self._string_value is not None:
            s = self._string_value
            return ValueSet([
                create_simple_object(self.parent_context.inference_state, s)
            ])
        return compiled.get_string_value_set(self.parent_context.inference_state)


class SubModuleDictMixin:
    @inference_state_method_cache()
    def sub_modules_dict(self):
        """
        Lists modules in the directory of this module (if this module is a
        package).
        """
        names = {}
        if self.is_package():
            mods = self.inference_state.compiled_subprocess.iter_module_names(
                self.py__path__()
            )
            for name in mods:
                # It's obviously a relative import to the current module.
                names[name] = SubModuleName(self.as_context(), name)

        # In the case of an import like `from x.` we don't need to
        # add all the variables, this is only about submodules.
        return names


class ModuleMixin(SubModuleDictMixin):
    _module_name_class = ModuleName

    def get_filters(self, origin_scope=None):
        yield MergedFilter(
            ParserTreeFilter(
                parent_context=self.as_context(),
                origin_scope=origin_scope
            ),
            GlobalNameFilter(self.as_context()),
        )
        yield DictFilter(self.sub_modules_dict())
        yield DictFilter(self._module_attributes_dict())
        yield from self.iter_star_filters()

    def py__class__(self):
        c, = values_from_qualified_names(self.inference_state, 'types', 'ModuleType')
        return c

    def is_module(self):
        return True

    def is_stub(self):
        return False

    @property  # type: ignore[misc]
    @inference_state_method_cache()
    def name(self):
        return self._module_name_class(self, self.string_names[-1])

    @inference_state_method_cache()
    def _module_attributes_dict(self):
        names = ['__package__', '__doc__', '__name__']
        # All the additional module attributes are strings.
        dct = dict((n, _ModuleAttributeName(self, n)) for n in names)
        path = self.py__file__()
        if path is not None:
            dct['__file__'] = _ModuleAttributeName(self, '__file__', str(path))
        return dct

    def iter_star_filters(self):
        for star_module in self.star_imports():
            f = next(star_module.get_filters(), None)
            assert f is not None
            yield f

    # I'm not sure if the star import cache is really that effective anymore
    # with all the other really fast import caches. Recheck. Also we would need
    # to push the star imports into InferenceState.module_cache, if we reenable this.
    @inference_state_method_cache([])
    def star_imports(self):
        from jedi.inference.imports import Importer

        modules = []
        module_context = self.as_context()
        for i in self.tree_node.iter_imports():
            if i.is_star_import():
                new = Importer(
                    self.inference_state,
                    import_path=i.get_paths()[-1],
                    module_context=module_context,
                    level=i.level
                ).follow()

                for module in new:
                    if isinstance(module, ModuleValue):
                        modules += module.star_imports()
                modules += new
        return modules

    def get_qualified_names(self):
        """
        A module doesn't have a qualified name, but it's important to note that
        it's reachable and not `None`. With this information we can add
        qualified names on top for all value children.
        """
        return ()


class ModuleValue(ModuleMixin, TreeValue):
    api_type = 'module'

    def __init__(self, inference_state, module_node, code_lines, file_io=None,
                 string_names=None, is_package=False):
        super().__init__(
            inference_state,
            parent_context=None,
            tree_node=module_node
        )
        self.file_io = file_io
        if file_io is None:
            self._path: Optional[Path] = None
        else:
            self._path = file_io.path
        self.string_names = string_names  # Optional[Tuple[str, ...]]
        self.code_lines = code_lines
        self._is_package = is_package

    def is_stub(self):
        if self._path is not None and self._path.suffix == '.pyi':
            # Currently this is the way how we identify stubs when e.g. goto is
            # used in them. This could be changed if stubs would be identified
            # sooner and used as StubModuleValue.
            return True
        return super().is_stub()

    def py__name__(self):
        if self.string_names is None:
            return None
        return '.'.join(self.string_names)

    def py__file__(self) -> Optional[Path]:
        """
        In contrast to Python's __file__ can be None.
        """
        if self._path is None:
            return None

        return self._path.absolute()

    def is_package(self):
        return self._is_package

    def py__package__(self):
        if self.string_names is None:
            return []

        if self._is_package:
            return self.string_names
        return self.string_names[:-1]

    def py__path__(self):
        """
        In case of a package, this returns Python's __path__ attribute, which
        is a list of paths (strings).
        Returns None if the module is not a package.
        """
        if not self._is_package:
            return None

        # A namespace package is typically auto generated and ~10 lines long.
        first_few_lines = ''.join(self.code_lines[:50])
        # these are strings that need to be used for namespace packages,
        # the first one is ``pkgutil``, the second ``pkg_resources``.
        options = ('declare_namespace(__name__)', 'extend_path(__path__')
        if options[0] in first_few_lines or options[1] in first_few_lines:
            # It is a namespace, now try to find the rest of the
            # modules on sys_path or whatever the search_path is.
            paths = set()
            for s in self.inference_state.get_sys_path():
                other = os.path.join(s, self.name.string_name)
                if os.path.isdir(other):
                    paths.add(other)
            if paths:
                return list(paths)
            # Nested namespace packages will not be supported. Nobody ever
            # asked for it and in Python 3 they are there without using all the
            # crap above.

        # Default to the of this file.
        file = self.py__file__()
        assert file is not None  # Shouldn't be a package in the first place.
        return [os.path.dirname(file)]

    def _as_context(self):
        return ModuleContext(self)

    def __repr__(self):
        return "<%s: %s@%s-%s is_stub=%s>" % (
            self.__class__.__name__, self.py__name__(),
            self.tree_node.start_pos[0], self.tree_node.end_pos[0],
            self.is_stub()
        )



import sys
from typing import Optional

def version() -> str: ...

if sys.version_info >= (3, 0):
    def bootstrap(
        *,
        root: Optional[str] = ...,
        upgrade: bool = ...,
        user: bool = ...,
        altinstall: bool = ...,
        default_pip: bool = ...,
        verbosity: int = ...,
    ) -> None: ...

else:
    def bootstrap(
        root: Optional[str] = ...,
        upgrade: bool = ...,
        user: bool = ...,
        altinstall: bool = ...,
        default_pip: bool = ...,
        verbosity: int = ...,
    ) -> None: ...



from typing import Any, Iterable, Optional, Text

def colored(
    text: Text, color: Optional[Text] = ..., on_color: Optional[Text] = ..., attrs: Optional[Iterable[Text]] = ...
) -> Text: ...
def cprint(
    text: Text, color: Optional[Text] = ..., on_color: Optional[Text] = ..., attrs: Optional[Iterable[Text]] = ..., **kwargs: Any
) -> None: ...



from ._base import (
    ALL_COMPLETED as ALL_COMPLETED,
    FIRST_COMPLETED as FIRST_COMPLETED,
    FIRST_EXCEPTION as FIRST_EXCEPTION,
    CancelledError as CancelledError,
    Executor as Executor,
    Future as Future,
    TimeoutError as TimeoutError,
    as_completed as as_completed,
    wait as wait,
)
from .process import ProcessPoolExecutor as ProcessPoolExecutor
from .thread import ThreadPoolExecutor as ThreadPoolExecutor



from typing import Any, Dict, Optional, Text, Type

from boto.connection import AWSAuthConnection
from boto.exception import BotoClientError

from .bucket import Bucket

def check_lowercase_bucketname(n): ...
def assert_case_insensitive(f): ...

class _CallingFormat:
    def get_bucket_server(self, server, bucket): ...
    def build_url_base(self, connection, protocol, server, bucket, key: str = ...): ...
    def build_host(self, server, bucket): ...
    def build_auth_path(self, bucket, key: str = ...): ...
    def build_path_base(self, bucket, key: str = ...): ...

class SubdomainCallingFormat(_CallingFormat):
    def get_bucket_server(self, server, bucket): ...

class VHostCallingFormat(_CallingFormat):
    def get_bucket_server(self, server, bucket): ...

class OrdinaryCallingFormat(_CallingFormat):
    def get_bucket_server(self, server, bucket): ...
    def build_path_base(self, bucket, key: str = ...): ...

class ProtocolIndependentOrdinaryCallingFormat(OrdinaryCallingFormat):
    def build_url_base(self, connection, protocol, server, bucket, key: str = ...): ...

class Location:
    DEFAULT: str
    EU: str
    EUCentral1: str
    USWest: str
    USWest2: str
    SAEast: str
    APNortheast: str
    APSoutheast: str
    APSoutheast2: str
    CNNorth1: str

class NoHostProvided: ...
class HostRequiredError(BotoClientError): ...

class S3Connection(AWSAuthConnection):
    DefaultHost: Any
    DefaultCallingFormat: Any
    QueryString: str
    calling_format: Any
    bucket_class: Type[Bucket]
    anon: Any
    def __init__(
        self,
        aws_access_key_id: Optional[Any] = ...,
        aws_secret_access_key: Optional[Any] = ...,
        is_secure: bool = ...,
        port: Optional[Any] = ...,
        proxy: Optional[Any] = ...,
        proxy_port: Optional[Any] = ...,
        proxy_user: Optional[Any] = ...,
        proxy_pass: Optional[Any] = ...,
        host: Any = ...,
        debug: int = ...,
        https_connection_factory: Optional[Any] = ...,
        calling_format: Any = ...,
        path: str = ...,
        provider: str = ...,
        bucket_class: Type[Bucket] = ...,
        security_token: Optional[Any] = ...,
        suppress_consec_slashes: bool = ...,
        anon: bool = ...,
        validate_certs: Optional[Any] = ...,
        profile_name: Optional[Any] = ...,
    ) -> None: ...
    def __iter__(self): ...
    def __contains__(self, bucket_name): ...
    def set_bucket_class(self, bucket_class: Type[Bucket]) -> None: ...
    def build_post_policy(self, expiration_time, conditions): ...
    def build_post_form_args(
        self,
        bucket_name,
        key,
        expires_in: int = ...,
        acl: Optional[Any] = ...,
        success_action_redirect: Optional[Any] = ...,
        max_content_length: Optional[Any] = ...,
        http_method: str = ...,
        fields: Optional[Any] = ...,
        conditions: Optional[Any] = ...,
        storage_class: str = ...,
        server_side_encryption: Optional[Any] = ...,
    ): ...
    def generate_url_sigv4(
        self,
        expires_in,
        method,
        bucket: str = ...,
        key: str = ...,
        headers: Optional[Dict[Text, Text]] = ...,
        force_http: bool = ...,
        response_headers: Optional[Dict[Text, Text]] = ...,
        version_id: Optional[Any] = ...,
        iso_date: Optional[Any] = ...,
    ): ...
    def generate_url(
        self,
        expires_in,
        method,
        bucket: str = ...,
        key: str = ...,
        headers: Optional[Dict[Text, Text]] = ...,
        query_auth: bool = ...,
        force_http: bool = ...,
        response_headers: Optional[Dict[Text, Text]] = ...,
        expires_in_absolute: bool = ...,
        version_id: Optional[Any] = ...,
    ): ...
    def get_all_buckets(self, headers: Optional[Dict[Text, Text]] = ...): ...
    def get_canonical_user_id(self, headers: Optional[Dict[Text, Text]] = ...): ...
    def get_bucket(self, bucket_name: Text, validate: bool = ..., headers: Optional[Dict[Text, Text]] = ...) -> Bucket: ...
    def head_bucket(self, bucket_name, headers: Optional[Dict[Text, Text]] = ...): ...
    def lookup(self, bucket_name, validate: bool = ..., headers: Optional[Dict[Text, Text]] = ...): ...
    def create_bucket(
        self, bucket_name, headers: Optional[Dict[Text, Text]] = ..., location: Any = ..., policy: Optional[Any] = ...
    ): ...
    def delete_bucket(self, bucket, headers: Optional[Dict[Text, Text]] = ...): ...
    def make_request(self, method, bucket: str = ..., key: str = ..., headers: Optional[Any] = ..., data: str = ..., query_args: Optional[Any] = ..., sender: Optional[Any] = ..., override_num_retries: Optional[Any] = ..., retry_handler: Optional[Any] = ..., *args, **kwargs): ...  # type: ignore # https://github.com/python/mypy/issues/1237



import sys
from typing import Any, ByteString, Sequence, Tuple, Type, TypeVar, Union, overload

from .descriptor import Descriptor, FieldDescriptor
from .internal.extension_dict import _ExtensionDict, _ExtensionFieldDescriptor

class Error(Exception): ...
class DecodeError(Error): ...
class EncodeError(Error): ...

_M = TypeVar("_M", bound=Message)  # message type (of self)

if sys.version_info < (3,):
    _Serialized = Union[bytes, buffer, unicode]
else:
    _Serialized = ByteString

class Message:
    DESCRIPTOR: Descriptor
    def __deepcopy__(self, memo=...): ...
    def __eq__(self, other_msg): ...
    def __ne__(self, other_msg): ...
    def MergeFrom(self: _M, other_msg: _M) -> None: ...
    def CopyFrom(self: _M, other_msg: _M) -> None: ...
    def Clear(self) -> None: ...
    def SetInParent(self) -> None: ...
    def IsInitialized(self) -> bool: ...
    def MergeFromString(self, serialized: _Serialized) -> int: ...
    def ParseFromString(self, serialized: _Serialized) -> int: ...
    def SerializeToString(self, deterministic: bool = ...) -> bytes: ...
    def SerializePartialToString(self, deterministic: bool = ...) -> bytes: ...
    def ListFields(self) -> Sequence[Tuple[FieldDescriptor, Any]]: ...
    # Dummy fallback overloads with FieldDescriptor are for backward compatibility with
    # mypy-protobuf <= 1.23. We can drop them a few months after 1.24 releases.
    @overload
    def HasExtension(self: _M, extension_handle: _ExtensionFieldDescriptor[_M, Any]) -> bool: ...
    @overload
    def HasExtension(self, extension_handle: FieldDescriptor) -> bool: ...
    @overload
    def ClearExtension(self: _M, extension_handle: _ExtensionFieldDescriptor[_M, Any]) -> None: ...
    @overload
    def ClearExtension(self, extension_handle: FieldDescriptor) -> None: ...
    def ByteSize(self) -> int: ...
    @classmethod
    def FromString(cls: Type[_M], s: _Serialized) -> _M: ...
    @property
    def Extensions(self: _M) -> _ExtensionDict[_M]: ...
    # Intentionally left out typing on these three methods, because they are
    # stringly typed and it is not useful to call them on a Message directly.
    # We prefer more specific typing on individual subclasses of Message
    # See https://github.com/dropbox/mypy-protobuf/issues/62 for details
    def HasField(self, field_name: Any) -> bool: ...
    def ClearField(self, field_name: Any) -> None: ...
    def WhichOneof(self, oneof_group: Any) -> Any: ...
    # TODO: check kwargs
    def __init__(self, **kwargs) -> None: ...



from typing import Any, Callable, Dict, Iterable, List, Optional, Sequence, Tuple, Type, TypeVar, Union, overload
from uuid import UUID

from django.db import models
from django.db.models.base import Model
from django.db.models.fields import Field
from django.db.models.query_utils import Q, PathInfo
from django.db.models.manager import RelatedManager
from django.db.models.expressions import Combinable
from django.db.models.fields.mixins import FieldCacheMixin
from django.db.models.fields.related_descriptors import (  # noqa: F401
    ForwardOneToOneDescriptor as ForwardOneToOneDescriptor,
    ForwardManyToOneDescriptor as ForwardManyToOneDescriptor,
    ManyToManyDescriptor as ManyToManyDescriptor,
    ReverseOneToOneDescriptor as ReverseOneToOneDescriptor,
    ReverseManyToOneDescriptor as ReverseManyToOneDescriptor,
)
from django.db.models.fields.reverse_related import (  # noqa: F401
    ForeignObjectRel as ForeignObjectRel,
    OneToOneRel as OneToOneRel,
    ManyToOneRel as ManyToOneRel,
    ManyToManyRel as ManyToManyRel,
)

_T = TypeVar("_T", bound=models.Model)
_F = TypeVar("_F", bound=models.Field)
_Choice = Tuple[Any, str]
_ChoiceNamedGroup = Tuple[str, Iterable[_Choice]]
_FieldChoices = Iterable[Union[_Choice, _ChoiceNamedGroup]]

_ValidatorCallable = Callable[..., None]
_ErrorMessagesToOverride = Dict[str, Any]

RECURSIVE_RELATIONSHIP_CONSTANT: str = ...

# __set__ value type
_ST = TypeVar("_ST")
# __get__ return type
_GT = TypeVar("_GT")

class RelatedField(FieldCacheMixin, Field[_ST, _GT]):
    one_to_many: bool = ...
    one_to_one: bool = ...
    many_to_many: bool = ...
    many_to_one: bool = ...
    related_model: Type[Model]
    opts: Any = ...
    def get_forward_related_filter(self, obj: Model) -> Dict[str, Union[int, UUID]]: ...
    def get_reverse_related_filter(self, obj: Model) -> Q: ...
    @property
    def swappable_setting(self) -> Optional[str]: ...
    def set_attributes_from_rel(self) -> None: ...
    def do_related_class(self, other: Type[Model], cls: Type[Model]) -> None: ...
    def get_limit_choices_to(self) -> Dict[str, int]: ...
    def related_query_name(self) -> str: ...
    @property
    def target_field(self) -> Field: ...

class ForeignObject(RelatedField[_ST, _GT]):
    def __init__(
        self,
        to: Union[Type[Model], str],
        on_delete: Callable[..., None],
        from_fields: Sequence[str],
        to_fields: Sequence[str],
        rel: Optional[ForeignObjectRel] = ...,
        related_name: Optional[str] = ...,
        related_query_name: Optional[str] = ...,
        limit_choices_to: Optional[Union[Dict[str, Any], Callable[[], Any]]] = ...,
        parent_link: bool = ...,
        db_constraint: bool = ...,
        swappable: bool = ...,
        verbose_name: Optional[str] = ...,
        name: Optional[str] = ...,
        primary_key: bool = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: bool = ...,
        db_index: bool = ...,
        default: Any = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        choices: Optional[_FieldChoices] = ...,
        help_text: str = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[_ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesToOverride] = ...,
    ): ...

class ForeignKey(ForeignObject[_ST, _GT]):
    _pyi_private_set_type: Union[Any, Combinable]
    _pyi_private_get_type: Any
    def __init__(
        self,
        to: Union[Type[Model], str],
        on_delete: Callable[..., None],
        to_field: Optional[str] = ...,
        related_name: Optional[str] = ...,
        related_query_name: Optional[str] = ...,
        limit_choices_to: Optional[Union[Dict[str, Any], Callable[[], Any], Q]] = ...,
        parent_link: bool = ...,
        db_constraint: bool = ...,
        verbose_name: Optional[Union[str, bytes]] = ...,
        name: Optional[str] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: bool = ...,
        db_index: bool = ...,
        default: Any = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Optional[_FieldChoices] = ...,
        help_text: str = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[_ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesToOverride] = ...,
    ): ...
    # class access
    @overload  # type: ignore
    def __get__(self, instance: None, owner) -> ForwardManyToOneDescriptor: ...
    # Model instance access
    @overload
    def __get__(self, instance: Model, owner) -> _GT: ...
    # non-Model instances
    @overload
    def __get__(self: _F, instance, owner) -> _F: ...

class OneToOneField(RelatedField[_ST, _GT]):
    _pyi_private_set_type: Union[Any, Combinable]
    _pyi_private_get_type: Any
    def __init__(
        self,
        to: Union[Type[Model], str],
        on_delete: Any,
        to_field: Optional[str] = ...,
        related_name: Optional[str] = ...,
        related_query_name: Optional[str] = ...,
        limit_choices_to: Optional[Union[Dict[str, Any], Callable[[], Any], Q]] = ...,
        parent_link: bool = ...,
        db_constraint: bool = ...,
        verbose_name: Optional[Union[str, bytes]] = ...,
        name: Optional[str] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: bool = ...,
        db_index: bool = ...,
        default: Any = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Optional[_FieldChoices] = ...,
        help_text: str = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[_ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesToOverride] = ...,
    ): ...
    # class access
    @overload  # type: ignore
    def __get__(self, instance: None, owner) -> ForwardOneToOneDescriptor: ...
    # Model instance access
    @overload
    def __get__(self, instance: Model, owner) -> _GT: ...
    # non-Model instances
    @overload
    def __get__(self: _F, instance, owner) -> _F: ...

class ManyToManyField(RelatedField[_ST, _GT]):
    _pyi_private_set_type: Sequence[Any]
    _pyi_private_get_type: RelatedManager[Any]

    rel_class: Any = ...
    description: Any = ...
    has_null_arg: Any = ...
    swappable: bool = ...
    def __init__(
        self,
        to: Union[Type[_T], str],
        related_name: Optional[str] = ...,
        related_query_name: Optional[str] = ...,
        limit_choices_to: Optional[Union[Dict[str, Any], Callable[[], Any], Q]] = ...,
        symmetrical: Optional[bool] = ...,
        through: Optional[Union[str, Type[Model]]] = ...,
        through_fields: Optional[Tuple[str, str]] = ...,
        db_constraint: bool = ...,
        db_table: Optional[str] = ...,
        swappable: bool = ...,
        verbose_name: Optional[Union[str, bytes]] = ...,
        name: Optional[str] = ...,
        primary_key: bool = ...,
        max_length: Optional[int] = ...,
        unique: bool = ...,
        blank: bool = ...,
        null: bool = ...,
        db_index: bool = ...,
        default: Any = ...,
        editable: bool = ...,
        auto_created: bool = ...,
        serialize: bool = ...,
        unique_for_date: Optional[str] = ...,
        unique_for_month: Optional[str] = ...,
        unique_for_year: Optional[str] = ...,
        choices: Optional[_FieldChoices] = ...,
        help_text: str = ...,
        db_column: Optional[str] = ...,
        db_tablespace: Optional[str] = ...,
        validators: Iterable[_ValidatorCallable] = ...,
        error_messages: Optional[_ErrorMessagesToOverride] = ...,
    ) -> None: ...
    # class access
    @overload  # type: ignore
    def __get__(self, instance: None, owner) -> ManyToManyDescriptor: ...
    # Model instance access
    @overload
    def __get__(self, instance: Model, owner) -> _GT: ...
    # non-Model instances
    @overload
    def __get__(self: _F, instance, owner) -> _F: ...
    def get_path_info(self, filtered_relation: None = ...) -> List[PathInfo]: ...
    def get_reverse_path_info(self, filtered_relation: None = ...) -> List[PathInfo]: ...
    def contribute_to_related_class(self, cls: Type[Model], related: RelatedField) -> None: ...
    def m2m_db_table(self) -> str: ...
    def m2m_column_name(self) -> str: ...
    def m2m_reverse_name(self) -> str: ...
    def m2m_reverse_field_name(self) -> str: ...
    def m2m_target_field_name(self) -> str: ...
    def m2m_reverse_target_field_name(self) -> str: ...

def create_many_to_many_intermediary_model(field: Type[Field], klass: Type[Model]) -> Type[Model]: ...



from datetime import datetime, date
from typing import Any, Optional, Union

from django.utils.timezone import FixedOffset

re_formatchars: Any
re_escaped: Any

class Formatter:
    def format(self, formatstr: str) -> str: ...

class TimeFormat(Formatter):
    data: Union[datetime, str] = ...
    timezone: Optional[FixedOffset] = ...
    def __init__(self, obj: Union[datetime, str]) -> None: ...
    def a(self) -> str: ...
    def A(self) -> str: ...
    def B(self) -> None: ...
    def e(self) -> str: ...
    def f(self) -> Union[int, str]: ...
    def g(self) -> int: ...
    def G(self) -> int: ...
    def h(self) -> str: ...
    def H(self) -> str: ...
    def i(self) -> str: ...
    def O(self) -> str: ...
    def P(self) -> str: ...
    def s(self) -> str: ...
    def T(self) -> str: ...
    def u(self) -> str: ...
    def Z(self) -> Union[int, str]: ...

class DateFormat(TimeFormat):
    data: Union[datetime, str]
    timezone: Optional[FixedOffset]
    year_days: Any = ...
    def b(self): ...
    def c(self) -> str: ...
    def d(self) -> str: ...
    def D(self): ...
    def E(self): ...
    def F(self): ...
    def I(self) -> str: ...
    def j(self) -> int: ...
    def l(self): ...
    def L(self) -> bool: ...
    def m(self) -> str: ...
    def M(self) -> str: ...
    def n(self) -> int: ...
    def N(self): ...
    def o(self) -> int: ...
    def r(self) -> str: ...
    def S(self) -> str: ...
    def t(self) -> str: ...
    def U(self) -> int: ...
    def w(self) -> int: ...
    def W(self) -> int: ...
    def y(self) -> str: ...
    def Y(self) -> int: ...
    def z(self) -> int: ...

def format(value: Union[datetime, str, date], format_string: str) -> str: ...
def time_format(value: Union[datetime, str], format_string: str) -> str: ...



class InputState(object):
    PURE_ASCII: int
    ESC_ASCII: int
    HIGH_BYTE: int

class LanguageFilter(object):
    CHINESE_SIMPLIFIED: int
    CHINESE_TRADITIONAL: int
    JAPANESE: int
    KOREAN: int
    NON_CJK: int
    ALL: int
    CHINESE: int
    CJK: int

class ProbingState(object):
    DETECTING: int
    FOUND_IT: int
    NOT_ME: int

class MachineState(object):
    START: int
    ERROR: int
    ITS_ME: int

class SequenceLikelihood(object):
    NEGATIVE: int
    UNLIKELY: int
    LIKELY: int
    POSITIVE: int
    @classmethod
    def get_num_categories(cls) -> int: ...

class CharacterCategory(object):
    UNDEFINED: int
    LINE_BREAK: int
    SYMBOL: int
    DIGIT: int
    CONTROL: int



from typing import Any

from django.contrib.admin.helpers import InlineAdminForm
from django.template.base import Parser, Token
from django.template.context import Context, RequestContext

from .base import InclusionAdminNode

register: Any

def prepopulated_fields_js(context: RequestContext) -> RequestContext: ...
def prepopulated_fields_js_tag(parser: Parser, token: Token) -> InclusionAdminNode: ...
def submit_row(context: RequestContext) -> Context: ...
def submit_row_tag(parser: Parser, token: Token) -> InclusionAdminNode: ...
def change_form_object_tools_tag(parser: Parser, token: Token) -> InclusionAdminNode: ...
def cell_count(inline_admin_form: InlineAdminForm) -> int: ...



from _typeshed import SupportsRead
from typing import IO, Any, Mapping, Optional, Sequence, Text, Union

from yaml.constructor import BaseConstructor, Constructor, SafeConstructor
from yaml.representer import BaseRepresenter, Representer, SafeRepresenter
from yaml.resolver import BaseResolver, Resolver
from yaml.serializer import Serializer

_Readable = SupportsRead[Union[Text, bytes]]

class CParser:
    def __init__(self, stream: Union[str, bytes, _Readable]) -> None: ...

class CBaseLoader(CParser, BaseConstructor, BaseResolver):
    def __init__(self, stream: Union[str, bytes, _Readable]) -> None: ...

class CLoader(CParser, SafeConstructor, Resolver):
    def __init__(self, stream: Union[str, bytes, _Readable]) -> None: ...

class CSafeLoader(CParser, SafeConstructor, Resolver):
    def __init__(self, stream: Union[str, bytes, _Readable]) -> None: ...

class CDangerLoader(CParser, Constructor, Resolver): ...  # undocumented

class CEmitter(object):
    def __init__(
        self,
        stream: IO[Any],
        canonical: Optional[Any] = ...,
        indent: Optional[int] = ...,
        width: Optional[int] = ...,
        allow_unicode: Optional[Any] = ...,
        line_break: Optional[str] = ...,
        encoding: Optional[Text] = ...,
        explicit_start: Optional[Any] = ...,
        explicit_end: Optional[Any] = ...,
        version: Optional[Sequence[int]] = ...,
        tags: Optional[Mapping[Text, Text]] = ...,
    ) -> None: ...

class CBaseDumper(CEmitter, BaseRepresenter, BaseResolver):
    def __init__(
        self,
        stream: IO[Any],
        default_style: Optional[str] = ...,
        default_flow_style: Optional[bool] = ...,
        canonical: Optional[Any] = ...,
        indent: Optional[int] = ...,
        width: Optional[int] = ...,
        allow_unicode: Optional[Any] = ...,
        line_break: Optional[str] = ...,
        encoding: Optional[Text] = ...,
        explicit_start: Optional[Any] = ...,
        explicit_end: Optional[Any] = ...,
        version: Optional[Sequence[int]] = ...,
        tags: Optional[Mapping[Text, Text]] = ...,
    ) -> None: ...

class CDumper(CEmitter, SafeRepresenter, Resolver): ...

CSafeDumper = CDumper

class CDangerDumper(CEmitter, Serializer, Representer, Resolver): ...  # undocumented



from logging import Logger
from typing import Any, Callable, Dict, Optional, Sequence, Tuple, Type, TypeVar, Union

_T = TypeVar("_T", bound=Callable[..., Any])
_Decorator = Callable[[_T], _T]
_R = TypeVar("_R")

def retry_call(
    f: Callable[..., _R],
    fargs: Optional[Sequence[Any]] = ...,
    fkwargs: Optional[Dict[str, Any]] = ...,
    exceptions: Union[Type[Exception], Tuple[Type[Exception], ...]] = ...,
    tries: int = ...,
    delay: float = ...,
    max_delay: Optional[float] = ...,
    backoff: float = ...,
    jitter: Union[Tuple[float, float], float] = ...,
    logger: Optional[Logger] = ...,
) -> _R: ...
def retry(
    exceptions: Union[Type[Exception], Tuple[Type[Exception], ...]] = ...,
    tries: int = ...,
    delay: float = ...,
    max_delay: Optional[float] = ...,
    backoff: float = ...,
    jitter: Union[Tuple[float, float], float] = ...,
    logger: Optional[Logger] = ...,
) -> _Decorator: ...



from typing import Any, Callable, Optional, Type, TypeVar, overload
from typing_extensions import Literal

from .classic import ClassicAdapter

_F = TypeVar("_F", bound=Callable[..., Any])

class SphinxAdapter(ClassicAdapter):
    directive: Literal["versionadded", "versionchanged", "deprecated"]
    reason: str
    version: str
    action: Optional[str]
    category: Type[DeprecationWarning]
    def __init__(
        self,
        directive: Literal["versionadded", "versionchanged", "deprecated"],
        reason: str = ...,
        version: str = ...,
        action: Optional[str] = ...,
        category: Type[DeprecationWarning] = ...,
    ) -> None: ...
    def __call__(self, wrapped: _F) -> Callable[[_F], _F]: ...

def versionadded(reason: str = ..., version: str = ...) -> Callable[[_F], _F]: ...
def versionchanged(reason: str = ..., version: str = ...) -> Callable[[_F], _F]: ...
@overload
def deprecated(__wrapped: _F) -> _F: ...
@overload
def deprecated(
    reason: str = ..., *, version: str = ..., action: Optional[str] = ..., category: Optional[Type[DeprecationWarning]] = ...
) -> Callable[[_F], _F]: ...



from typing import Any, Generator

class Message:
    preamble = ...
    epilogue = ...
    defects = ...
    def __init__(self): ...
    def as_string(self, unixfrom=...): ...
    def is_multipart(self) -> bool: ...
    def set_unixfrom(self, unixfrom) -> None: ...
    def get_unixfrom(self): ...
    def attach(self, payload) -> None: ...
    def get_payload(self, i=..., decode: bool = ...): ...
    def set_payload(self, payload, charset=...) -> None: ...
    def set_charset(self, charset): ...
    def get_charset(self): ...
    def __len__(self): ...
    def __getitem__(self, name): ...
    def __setitem__(self, name, val) -> None: ...
    def __delitem__(self, name) -> None: ...
    def __contains__(self, name): ...
    def has_key(self, name) -> bool: ...
    def keys(self): ...
    def values(self): ...
    def items(self): ...
    def get(self, name, failobj=...): ...
    def get_all(self, name, failobj=...): ...
    def add_header(self, _name, _value, **_params) -> None: ...
    def replace_header(self, _name, _value) -> None: ...
    def get_content_type(self): ...
    def get_content_maintype(self): ...
    def get_content_subtype(self): ...
    def get_default_type(self): ...
    def set_default_type(self, ctype) -> None: ...
    def get_params(self, failobj=..., header=..., unquote: bool = ...): ...
    def get_param(self, param, failobj=..., header=..., unquote: bool = ...): ...
    def set_param(self, param, value, header=..., requote: bool = ..., charset=..., language=...) -> None: ...
    def del_param(self, param, header=..., requote: bool = ...): ...
    def set_type(self, type, header=..., requote: bool = ...): ...
    def get_filename(self, failobj=...): ...
    def get_boundary(self, failobj=...): ...
    def set_boundary(self, boundary) -> None: ...
    def get_content_charset(self, failobj=...): ...
    def get_charsets(self, failobj=...): ...
    def walk(self) -> Generator[Any, Any, Any]: ...
